@@ -306,7 +306,7 @@ main(
 
     if (am_has_feature(g_options->features, fe_req_xml)) {
 	char  *errmsg = NULL;
-	dle_t *dles, *dle;
+	dle_t *dles, *dle, *dle_next;
 
 	dles = amxml_parse_node_FILE(stdin, &errmsg);
 	if (errmsg) {
@@ -329,6 +329,10 @@ main(
 	    run_client_scripts(EXECUTE_ON_POST_HOST_AMCHECK, g_options, dle,
 			       stdout);
 	}
+	for (dle = dles; dle != NULL; dle = dle_next) {
+	    dle_next = dle->next;
+	    free_dle(dle);
+	}
     }
 
     check_overall();
