@@ -189,11 +189,12 @@ sub inventory {
 	    if ($label) {
 		$s->{'label'} = $self->_get_slot_label($slot);
 		$s->{'f_type'} = "".$Amanda::Header::F_TAPESTART;
+		$s->{'device_status'} = "".$DEVICE_STATUS_SUCCESS;
 	    } else {
 		$s->{'label'} = undef;
 		$s->{'f_type'} = "".$Amanda::Header::F_EMPTY;
+		$s->{'device_status'} = "".$DEVICE_STATUS_VOLUME_UNLABELED;
 	    }
-	    $s->{'device_status'} = "".$DEVICE_STATUS_SUCCESS;
 	    $s->{'current'} = 1 if $slot eq $current;
 	    push @inventory, $s;
 	}
@@ -325,7 +326,7 @@ sub _alloc_drive {
 }
 
 # Internal function to enumerate all available slots.  Slots are described by
-# integers.
+# strings.
 sub _all_slots {
     my ($self) = @_;
     my $dir = _quote_glob($self->{'dir'});
@@ -337,7 +338,7 @@ sub _all_slots {
 	push @slots, $slot + 0;
     }
 
-    return sort @slots;
+    return map { "$_"} sort { $a <=> $b } @slots;
 }
 
 # Internal function to determine whether a slot exists.
@@ -376,7 +377,7 @@ sub _is_slot_in_use {
 
 	    #check if process is alive
 	    my $pid = $state->{drives}->{$drive}->{pid};
-	    if (defined $pid && !Amanda::Util::is_pid_alive($pid)) {
+	    if (!defined $pid or !Amanda::Util::is_pid_alive($pid)) {
 		unlink("$drive/data")
 		    or warn("Could not unlink '$drive/data': $!");
 		rmdir("$drive")
