@@ -86,5 +86,12 @@ sub new {
     return $self;
 }
 
+sub _set_mesgout {
+    my $self = shift;
+
+    my $mesgout_fd;
+    open ($mesgout_fd, '>&=1') || die("Can't open mesgout_fd: $!");
+    $self->{mesgout} = $mesgout_fd;
+}
 
 1;
