@@ -44,12 +44,6 @@
 #include "holding.h"
 #include "timestamp.h"
 
-#define chunker_debug(i, ...) do {	\
-	if ((i) <= debug_chunker) {	\
-	    dbprintf(__VA_ARGS__);	\
-	}				\
-} while (0)
-
 #ifndef SEEK_SET
 #define SEEK_SET 0
 #endif
@@ -102,9 +96,14 @@ static ssize_t write_tapeheader(int, dumpfile_t *);
 static void databuf_init(struct databuf *, int, char *, off_t, off_t);
 static int databuf_flush(struct databuf *);
 
-static int startup_chunker(char *, off_t, off_t, struct databuf *, int *);
-static int do_chunk(int, struct databuf *, int);
+static int startup_chunker(char *, off_t, off_t, struct databuf *, int *, int *);
+static int do_chunk(int, struct databuf *, int, int);
 
+/* we use a function pointer for full_write, so that we can "shim" in
+ * full_write_with_fake_enospc for testing */
+static size_t (*db_full_write)(int fd, const void *buf, size_t count);
+static size_t full_write_with_fake_enospc(int fd, const void *buf, size_t count);
+static off_t fake_enospc_at_byte = -1;
 
 int
 main(
@@ -123,6 +122,7 @@ main(
     config_overrides_t *cfg_ovr = NULL;
     char *cfg_opt = NULL;
     char *m;
+    int header_socket;
     int data_socket;
 
     /*
@@ -189,6 +189,18 @@ main(
 	log_add(L_INFO, "%s pid %ld", get_pname(), (long)getpid());
 	error(_("Didn't get START command"));
     }
+    free_cmdargs(cmdargs);
+
+    /* set up a fake ENOSPC for testing purposes.  Note that this counts
+     * headers as well as data written to disk. */
+    if (getenv("CHUNKER_FAKE_ENOSPC_AT")) {
+	char *env = getenv("CHUNKER_FAKE_ENOSPC_AT");
+	fake_enospc_at_byte = (off_t)atoi(env); /* these values are never > MAXINT */
+	db_full_write = full_write_with_fake_enospc;
+	g_debug("will trigger fake ENOSPC at byte %d", (int)fake_enospc_at_byte);
+    } else {
+	db_full_write = full_write;
+    }
 
 /*    do {*/
 	cmdargs = getcmd();
@@ -296,13 +308,13 @@ main(
 	    }
 
 	    if ((header_fd = startup_chunker(filename, use, chunksize, &db,
-					     &data_socket)) < 0) {
+					     &header_socket, &data_socket)) < 0) {
 		q = quote_string(vstrallocf(_("[chunker startup failed: %s]"), errstr));
 		putresult(TRYAGAIN, "%s %s\n", handle, q);
 		error("startup_chunker failed: %s", errstr);
 	    }
 	    command_in_transit = NULL;
-	    if (header_fd >= 0 && do_chunk(header_fd, &db, data_socket)) {
+	    if (header_fd >= 0 && do_chunk(header_fd, &db, header_socket, data_socket)) {
 		char kb_str[NUM_STR_SIZE];
 		char kps_str[NUM_STR_SIZE];
 		double rt;
@@ -318,6 +330,7 @@ main(
 		m = vstrallocf("[%s]", errstr);
 		q = quote_string(m);
 		amfree(m);
+		free_cmdargs(cmdargs);
 		if(command_in_transit != NULL) {
 		    cmdargs = command_in_transit;
 		    command_in_transit = NULL;
@@ -418,6 +431,7 @@ startup_chunker(
     off_t		use,
     off_t		chunksize,
     struct databuf *	db,
+    int                *headersocket,
     int                *datasocket)
 {
     int header_fd, outfd;
@@ -463,7 +477,6 @@ startup_chunker(
 	aclose(data_socket);
 	return -1;
     }
-    aclose(header_socket);
 
     tmp_filename = vstralloc(filename, ".tmp", NULL);
     pc = strrchr(tmp_filename, '/');
@@ -493,6 +506,7 @@ startup_chunker(
     amfree(tmp_filename);
     databuf_init(db, outfd, filename, use, chunksize);
     db->filename_seq++;
+    *headersocket = header_socket;
     *datasocket = data_socket;
     return header_fd;
 }
@@ -501,6 +515,7 @@ static int
 do_chunk(
     int			header_fd,
     struct databuf *	db,
+    int                 header_socket,
     int                 data_socket)
 {
     size_t nread;
@@ -519,6 +534,8 @@ do_chunk(
      * chunk code will rewrite it.
      */
     nread = full_read(header_fd, header_buf, SIZEOF(header_buf));
+    aclose(header_fd);
+    aclose(header_socket);
     if (nread != sizeof(header_buf)) {
 	if(errno != 0) {
 	    errstr = vstrallocf(_("cannot read header: %s"), strerror(errno));
@@ -556,7 +573,6 @@ do_chunk(
 	aclose(data_socket);
 	return 0;
     }
-    aclose(data_socket);
 
     /*
      * We've written the file header.  Now, just write data until the
@@ -567,12 +583,16 @@ do_chunk(
 	db->datain += nread;
 	while(db->dataout < db->datain) {
 	    if(!databuf_flush(db)) {
+		aclose(data_fd);
+		aclose(data_socket);
 		return 0;
 	    }
 	}
     }
     while(db->dataout < db->datain) {
 	if(!databuf_flush(db)) {
+	    aclose(data_fd);
+	    aclose(data_socket);
 	    return 0;
 	}
     }
@@ -580,6 +600,8 @@ do_chunk(
 	dumpsize += (off_t)1;			/* count partial final KByte */
 	filesize += (off_t)1;
     }
+    aclose(data_fd);
+    aclose(data_socket);
     return 1;
 }
 
@@ -780,10 +802,14 @@ databuf_flush(
 
 	    aclose(newfd);
 	    if(save_errno == ENOSPC) {
+		if (unlink(tmp_filename) < 0) {
+		    g_debug("could not delete '%s'; ignoring", tmp_filename);
+		}
 		putresult(NO_ROOM, "%s %lld\n", handle, 
 			  (long long)(db->use+db->split_size-dumpsize));
 		db->use = (off_t)0;			/* force RQ_MORE DISK */
 		db->split_size = dumpsize;
+		file.type = save_type;
 		continue;
 	    }
 	    m = vstrallocf(_("write_tapeheader file %s: %s"),
@@ -864,7 +890,7 @@ databuf_flush(
      * Write out the buffer
      */
     size_to_write = (size_t)(db->datain - db->dataout);
-    written = full_write(db->fd, db->dataout, size_to_write);
+    written = db_full_write(db->fd, db->dataout, size_to_write);
     if (written > 0) {
 	db->dataout += written;
 	dumpbytes += (off_t)written;
@@ -927,7 +953,7 @@ write_tapeheader(
     if (!buffer) /* this shouldn't happen */
 	error(_("header does not fit in %zd bytes"), (size_t)DISK_BLOCK_BYTES);
 
-    written = full_write(outfd, buffer, DISK_BLOCK_BYTES);
+    written = db_full_write(outfd, buffer, DISK_BLOCK_BYTES);
     amfree(buffer);
     if(written == DISK_BLOCK_BYTES) return 0;
 
@@ -937,3 +963,41 @@ write_tapeheader(
 
     return (ssize_t)-1;
 }
+
+static size_t
+full_write_with_fake_enospc(
+    int fd,
+    const void *buf,
+    size_t count)
+{
+    size_t rc;
+
+    //g_debug("HERE %zd %zd", count, (size_t)fake_enospc_at_byte);
+
+    if (count <= (size_t)fake_enospc_at_byte) {
+	fake_enospc_at_byte -= count;
+	return full_write(fd, buf, count);
+    }
+
+    /* if we get here, the caller has requested a size that is less
+     * than fake_enospc_at_byte. */
+    count = fake_enospc_at_byte;
+    g_debug("returning fake ENOSPC");
+
+    if (fake_enospc_at_byte) {
+	rc = full_write(fd, buf, fake_enospc_at_byte);
+	if (rc == (size_t)fake_enospc_at_byte) {
+	    /* full_write succeeded, so fake a failure */
+	    errno = ENOSPC;
+	}
+    } else {
+	/* no bytes to write; just fake an error */
+	errno = ENOSPC;
+	rc = 0;
+    }
+
+    /* switch back to calling full_write directly */
+    fake_enospc_at_byte = -1;
+    db_full_write = full_write;
+    return rc;
+}
