@@ -24,9 +24,12 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: list_dir.h,v 1.3 1999/02/15 02:30:26 martinea Exp $
+ * $Id: list_dir.h,v 1.4 2006/05/25 01:47:20 johnfranks Exp $
  *
  */
+#ifndef LISTDIR_H
+#define LISTDIR_H
+
 
 typedef struct DIR_ITEM {
     DUMP_ITEM *dump;
@@ -34,7 +37,7 @@ typedef struct DIR_ITEM {
     struct DIR_ITEM *next;
 } DIR_ITEM;
 
-
-extern DIR_ITEM *get_dir_list P((void));
-extern void clear_dir_list P((void));
-extern int add_dir_list_item P((DUMP_ITEM *dump, char *path));
+extern struct DIR_ITEM *get_dir_list(void);
+extern void clear_dir_list(void);
+extern int add_dir_list_item(DUMP_ITEM *dump, const char *path);
+#endif	/* !LISTDIR_H */
