@@ -102,6 +102,8 @@ write_tapelist(
 	g_fprintf(tapef, "%s %s", tp->datestamp, tp->label);
 	if(tp->reuse) g_fprintf(tapef, " reuse");
 	else g_fprintf(tapef, " no-reuse");
+	if (tp->barcode)
+	    g_fprintf(tapef, " BARCODE:%s", tp->barcode);
 	if (tp->comment)
 	    g_fprintf(tapef, " #%s", tp->comment);
 	g_fprintf(tapef, "\n");
@@ -396,6 +398,16 @@ parse_tapeline(
 	s[-1] = '\0';
 	skip_whitespace(s, ch);
     }
+    if (strncmp_const(s - 1, "BARCODE:") == 0) {
+	s1 = s - 1 + 8;
+	skip_non_whitespace(s, ch);
+	s[-1] = '\0';
+	skip_whitespace(s, ch);
+	tp->barcode = stralloc(s1);
+    } else {
+	tp->barcode = NULL;
+    }
+
 
     if (*(s - 1) == '#') {
 	tp->comment = stralloc(s); /* skip leading '#' */
