@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2008,2009 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2008, 2009, 2010 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -19,8 +19,8 @@
  * Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
  */
 
-#include "amxfer.h"
 #include "amanda.h"
+#include "amxfer.h"
 
 /*
  * Class declaration
@@ -112,7 +112,7 @@ xfer_source_fd(
 
     /* we read from a *copy* of this file descriptor, as the downstream element
      * will close output_fd on EOF */
-    elt->output_fd = dup(fd);
+    g_assert(xfer_element_swap_output_fd(elt, dup(fd)) == -1);
 
     return elt;
 }
